#include "DecayEvents.h"
#include "DecayUtil.h"
#include "HelasUtil.h"
#include <iostream>
#include <math.h>
using namespace std;

void DecayEvent(Event& orig_event, int location, DecayMode* dm,
		Vegas& Vegas, double bratio,
		FourVector boost)
{
  int nDaughters = dm->NumberDaughters();
  if(nDaughters == 2) {
    TwoBodyDecayDriver drv2(Vegas.RandomGenerator());
    drv2.SetDecayMode(dynamic_cast<TwoBodyDecayMode*>(dm));
    DecayEvent(orig_event,location,drv2,Vegas,bratio,boost);
  } else if(nDaughters == 3) {
    ThreeBodyDecayDriver drv3(Vegas.RandomGenerator());
    drv3.SetDecayMode(dynamic_cast<ThreeBodyDecayMode*>(dm));
    DecayEvent(orig_event,location,&drv3,Vegas,bratio,boost);
  } else {
    cout << "Oops. nDaughters should be 2 or 3 in DecayEvent....\n";
  }
}

void DecayEvent(Event& orig_event, int location, TwoBodyDecayDriver& driver, 
        Vegas& vegas, double bratio, FourVector boost)
{
  vector<EventParticle::EventParticle> orig_particles,dec_particles;
  EventParticle::EventParticle emptyparticle;
  //double cms[4];
  double weight=orig_event.xwgtup();
  double wgt=0.0;
  double mxwgt = vegas.MaxWeight();
  orig_particles=orig_event.get_particles();
  
  dec_particles.push_back(orig_particles[location]);
  dec_particles[0].setstatus(2);
  
  // don't pass twobodydecay eventually and get numdaughters
  // from general decaymode
  int numdaughters=2;

  for(int i=1;i<=numdaughters;i++)
  {
    dec_particles.push_back(emptyparticle);
  }

  // setting up all the id information for the new particles
  // cheating here for two body only again
  int maxcol=orig_event.max_color();
  for(int i=1;i<=numdaughters;i++)
  {
    dec_particles[i].setid(Particle::pl[driver.GetDecayMode()->ListDaughters()[i-1]].IDCode());
    dec_particles[i].setmoth1(location+1);
    //have to think about cases for 2 mothers?
    dec_particles[i].setmoth2(0);
    //Color labels: put this in another function!!
    //Specific to two body decays, some caveats
    if(Particle::pl[Particle::plFromID[dec_particles[i].idup()]].NColors()==1)
        {
        dec_particles[i].setcolor1(0);
        dec_particles[i].setcolor2(0);
        }
    else
        {
        if(dec_particles[0].icolup1()==0&&dec_particles[0].icolup2()==0)
            {
            // not sufficient for h->gg can add this but just getting basic 2 body color first
            // will need to distinguish gluon from "anti" gluon, can do this on simple case checker
            // but will have to specify to two body, I think this is ok since we will have more 
            // complication on 3-body color not even touched here
                //h->gg specific for 2 body
                if(Particle::pl[Particle::plFromID[dec_particles[i].idup()]].NColors()==8)
                {
                dec_particles[i].setcolor1(maxcol+i);
                dec_particles[i].setcolor2(maxcol+3-i);
                }
                else
                {
                    // asuming triplet here
                    if(dec_particles[i].idup()>0)
                    {
                    dec_particles[i].setcolor1(maxcol+1);
                    dec_particles[i].setcolor2(0);
                    }
                    else
                    {
                    dec_particles[i].setcolor2(maxcol+1);
                    dec_particles[i].setcolor1(0);
                    }
                }
            }
        else
            {
            //parent is colored: two cases 8 or 3, squirks? no...
            if(Particle::pl[Particle::plFromID[dec_particles[0].idup()]].NColors()==8)
                {
                if(dec_particles[i].idup()>0)
                    {
                    dec_particles[i].setcolor1(dec_particles[0].icolup1());
                    dec_particles[i].setcolor2(0);
                    }
                else
                    {
                    dec_particles[i].setcolor2(dec_particles[0].icolup2());
                    dec_particles[i].setcolor1(0);
                    }
                 }
            else
                {
		    string thisp = driver.GetDecayMode()->ListDaughters()[i-1];
		    string otherp = driver.GetDecayMode()->ListDaughters()[(3-i)-1];
		    int ncolthis = Particle::pl[thisp].NColors();
		    int ncolother = Particle::pl[otherp].NColors();
		    if(ncolthis == 3 && ncolother == 1) {
			dec_particles[i].setcolor1(dec_particles[0].icolup1());
			dec_particles[i].setcolor2(dec_particles[0].icolup2());
		    }
		    else if(ncolthis == 3 && ncolother == 8) {
			if(dec_particles[i].idup()>0) {
                            // 3 -> 3 + 8
			    dec_particles[i].setcolor1(maxcol+1);
			    dec_particles[i].setcolor2(0);
			} else {
                            // 3bar -> 3bar + 8
			    dec_particles[i].setcolor1(0);
			    dec_particles[i].setcolor2(maxcol+1);
			}
		    } else if(ncolthis == 8 && ncolother == 3) {
			if(dec_particles[3-i].idup()>0) {
                            // 3 -> 3 + 8
			    dec_particles[i].setcolor1(dec_particles[0].icolup1());
			    dec_particles[i].setcolor2(maxcol+1);
			} else {
                            // 3bar -> 3bar + 8
			    dec_particles[i].setcolor1(maxcol+1);
			    dec_particles[i].setcolor2(dec_particles[0].icolup2());
			}
		    }
                }   
            }
        }  
    dec_particles[i].setstatus(1);
    //P5 needs to be calculated so not set here
    //vtimup always this unless we add it to particle class... can let matt do studies of the ranch in principle...
    dec_particles[i].setvtimup(0.0);
    dec_particles[i].Helicity=driver.GetRandomHelicity(driver.GetDecayMode()->ListDaughters()[i-1]);
  }

  // need momentum, wgt
  /*
  for(int i=0;i<=3;i++)
  {
    cms[i]=double(-orig_particles[0].P[i]-orig_particles[1].P[i]);
  }
  cms[0]=-cms[0];
  for(int i=0;i<=3;i++)
  {
    cout << "orig_particles[location].P[" << i << "] = " << orig_particles[location].P[i] << "\n";
    cout << "cms[" << i << "] = " << cms[i] << "\n";
  }
  boostx_(orig_particles[location].P,cms,dec_particles[0].P);
  for(int i=0; i<=3; i++)
  {
    cout << "dec_particles[0].P[" << i << "] = " << dec_particles[0].P[i] << "\n";
  }
  */
  // boost to the reference frame we are given
  boostx_(orig_particles[location].P,boost.P,dec_particles[0].P);
  vector<double> X;

  bool done=false;
  while(!done)
  {
    double vegas_wgt=vegas.getRandomFromGrid(X);
    driver.SetMotherMomentum((FourVector) dec_particles[0]);
    wgt = driver.RunEvent(X, vegas_wgt);
    if(wgt != wgt) {
	cout << "Oops! Event weight is NaN, exiting to avoid infinite loop.\n";
	exit(0);
    }
    FourVector d1P = driver.Daughter1Momentum();
    FourVector d2P = driver.Daughter2Momentum();
    for(int i = 0; i < 4; i++) {
      dec_particles[1].P[i] = d1P.P[i];
      dec_particles[2].P[i] = d2P.P[i];
    }
    dec_particles[1].Helicity = d1P.Helicity;
    dec_particles[2].Helicity = d2P.Helicity;
    wgt *= vegas_wgt;
    //cout << "vegas weight is: "<< vegas_wgt  << endl;
    //cout << "evt. weight * vegas weight is: " << wgt << endl;
  //  cout << "Max weight is: " << mxwgt << endl;
    double random = (*(vegas.RandomGenerator()))();
//    cout << "Random is: " << random << endl;
    if(wgt/random > mxwgt)
    {
      done=true;
    }
  }
    
  // boost back to the lab frame
  for(int i = 1; i <= 3; i++) {
    boost.P[i] = -boost.P[i];
  }
  for(int i = 0; i <= numdaughters; i++) {
    boostx_(dec_particles[i].P,boost.P,dec_particles[i].P);
  }

  // modifying the original event to contain all the new information 
  // from the decay
  orig_particles[location]=dec_particles[0];
  for(int i=1;i<=numdaughters;i++)
  {
    //need to fix M() if its <0 return M <0 as per hep-ph/0109068
    //will a negative mass mess up anything else in the code?
    //this needs to be adressed at some point
    dec_particles[i].setP5(dec_particles[i].M());
    orig_particles.push_back(dec_particles[i]);
  }
  orig_event.add_particles(orig_particles);
  orig_event.setnup(orig_particles.size());

  orig_event.setxwgtup(weight*bratio);
  //cout << decaymode->Parent() <<  " -> " << decaymode->ListDaughters()[0] << ", " 
  //       << decaymode->ListDaughters()[1] << "\n";
  //parent location
  //cout << "location of particle is line number: " << location+1 << endl;       
  //
}


void DecayEvent(Event& orig_event, int location, ThreeBodyDecayDriver* driver, 
        Vegas& vegas, double bratio, FourVector boost)
{
  vector<EventParticle::EventParticle> orig_particles,dec_particles;
  EventParticle::EventParticle emptyparticle;
  //double cms[4];
  double weight=orig_event.xwgtup();
  double wgt=0.0;
  double mxwgt = vegas.MaxWeight();
  static bool first=true;
  if(first) {
      cout << "Max weight = " << mxwgt << "\n";
      first = false;
  }
  orig_particles=orig_event.get_particles();
  
  dec_particles.push_back(orig_particles[location]);
  dec_particles[0].setstatus(2);
  
  // don't pass twobodydecay eventually and get numdaughters
  // from general decaymode
  int numdaughters=3;

  for(int i=1;i<=numdaughters;i++)
  {
    dec_particles.push_back(emptyparticle);
  }

  // setting up all the id information for the new particles
  // cheating here for two body only again
  int maxcol=orig_event.max_color();
  for(int i=1;i<=numdaughters;i++)
  {
    dec_particles[i].setid(Particle::pl[driver->GetDecayMode()->ListDaughters()[i-1]].IDCode());
    dec_particles[i].setmoth1(location+1);
    //have to think about cases for 2 mothers?
    dec_particles[i].setmoth2(0);
    //Color labels: put this in another function!!
    //Specific to two body decays, some caveats
    if(Particle::pl[Particle::plFromID[dec_particles[i].idup()]].NColors()==1)
        {
        dec_particles[i].setcolor1(0);
        dec_particles[i].setcolor2(0);
        }
    else
        {
        if(dec_particles[0].icolup1()==0&&dec_particles[0].icolup2()==0)
            {
            // not sufficient for h->gg can add this but just getting basic 2 body color first
            // will need to distinguish gluon from "anti" gluon, can do this on simple case checker
            // but will have to specify to two body, I think this is ok since we will have more 
            // complication on 3-body color not even touched here
                //h->gg specific for 2 body
                if(Particle::pl[Particle::plFromID[dec_particles[i].idup()]].NColors()==8)
                {
                dec_particles[i].setcolor1(maxcol+i);
                dec_particles[i].setcolor2(maxcol+3-i);
                }
                else
                {
                    // asuming triplet here
                    if(dec_particles[i].idup()>0)
                    {
                    dec_particles[i].setcolor1(maxcol+1);
                    dec_particles[i].setcolor2(0);
                    }
                    else
                    {
                    dec_particles[i].setcolor2(maxcol+1);
                    dec_particles[i].setcolor1(0);
                    }
                }
            }
        else
            {
            //parent is colored: two cases 8 or 3, squirks? no...
            if(Particle::pl[Particle::plFromID[dec_particles[0].idup()]].NColors()==8)
                {
                if(dec_particles[i].idup()>0)
                    {
                    dec_particles[i].setcolor1(dec_particles[0].icolup1());
                    dec_particles[i].setcolor2(0);
                    }
                else
                    {
                    dec_particles[i].setcolor2(dec_particles[0].icolup2());
                    dec_particles[i].setcolor1(0);
                    }
                 }
            else
                {
		    string thisp = driver->GetDecayMode()->ListDaughters()[i-1];
		    string otherp = driver->GetDecayMode()->ListDaughters()[(3-i)-1];
		    int ncolthis = Particle::pl[thisp].NColors();
		    int ncolother = Particle::pl[otherp].NColors();
		    if(ncolthis == 3 && ncolother == 1) {
			dec_particles[i].setcolor1(dec_particles[0].icolup1());
			dec_particles[i].setcolor2(dec_particles[0].icolup2());
		    }
		    else if(ncolthis == 3 && ncolother == 8) {
			if(dec_particles[i].idup()>0) {
                            // 3 -> 3 + 8
			    dec_particles[i].setcolor1(maxcol+1);
			    dec_particles[i].setcolor2(0);
			} else {
                            // 3bar -> 3bar + 8
			    dec_particles[i].setcolor1(0);
			    dec_particles[i].setcolor2(maxcol+1);
			}
		    } else if(ncolthis == 8 && ncolother == 3) {
			if(dec_particles[3-i].idup()>0) {
                            // 3 -> 3 + 8
			    dec_particles[i].setcolor1(dec_particles[0].icolup1());
			    dec_particles[i].setcolor2(maxcol+1);
			} else {
                            // 3bar -> 3bar + 8
			    dec_particles[i].setcolor1(maxcol+1);
			    dec_particles[i].setcolor2(dec_particles[0].icolup2());
			}
		    }
                }   
            }
        }  
    dec_particles[i].setstatus(1);
    //P5 needs to be calculated so not set here
    //vtimup always this unless we add it to particle class... can let matt do studies of the ranch in principle...
    dec_particles[i].setvtimup(0.0);
    dec_particles[i].Helicity=driver->GetRandomHelicity(driver->GetDecayMode()->ListDaughters()[i-1]);
  }

  // need momentum, wgt
  /*
  for(int i=0;i<=3;i++)
  {
    cms[i]=double(-orig_particles[0].P[i]-orig_particles[1].P[i]);
  }
  cms[0]=-cms[0];
  for(int i=0;i<=3;i++)
  {
    cout << "orig_particles[location].P[" << i << "] = " << orig_particles[location].P[i] << "\n";
    cout << "cms[" << i << "] = " << cms[i] << "\n";
  }
  boostx_(orig_particles[location].P,cms,dec_particles[0].P);
  for(int i=0; i<=3; i++)
  {
    cout << "dec_particles[0].P[" << i << "] = " << dec_particles[0].P[i] << "\n";
  }
  */
  // boost to the reference frame we are given
  boostx_(orig_particles[location].P,boost.P,dec_particles[0].P);
  vector<double> X;

  bool done=false;
  while(!done)
  {
    double vegas_wgt=vegas.getRandomFromGrid(X);
    driver->SetMotherMomentum((FourVector) dec_particles[0]);
    wgt = driver->RunEvent(X, vegas_wgt);
    if(wgt != wgt) {
	cout << "Oops! Event weight is NaN, exiting to avoid infinite loop.\n";
	exit(0);
    }
    FourVector d1P = driver->Daughter1Momentum();
    FourVector d2P = driver->Daughter2Momentum();
    FourVector d3P = driver->Daughter3Momentum();
    for(int i = 0; i < 4; i++) {
      dec_particles[1].P[i] = d1P.P[i];
      dec_particles[2].P[i] = d2P.P[i];
      dec_particles[3].P[i] = d3P.P[i];
    }
    dec_particles[1].Helicity = d1P.Helicity;
    dec_particles[2].Helicity = d2P.Helicity;
    dec_particles[3].Helicity = d3P.Helicity;
    wgt *= vegas_wgt;
//    cout << "vegas weight is: "<< vegas_wgt  << endl;
//    cout << "evt. weight * vegas weight is: " << wgt << endl;
//    cout << "Max weight is: " << mxwgt << endl;
    double random = (*(vegas.RandomGenerator()))();
//    cout << "Random is: " << random << endl;
//    cout << "wgt/random = " << wgt/random << "\n";
    if(wgt/random > mxwgt)
    {
      done=true;
    }
  }
    
  // boost back to the lab frame
  for(int i = 1; i <= 3; i++) {
    boost.P[i] = -boost.P[i];
  }
  for(int i = 0; i <= numdaughters; i++) {
    boostx_(dec_particles[i].P,boost.P,dec_particles[i].P);
  }

  // modifying the original event to contain all the new information 
  // from the decay
  orig_particles[location]=dec_particles[0];
  for(int i=1;i<=numdaughters;i++)
  {
    //need to fix M() if its <0 return M <0 as per hep-ph/0109068
    //will a negative mass mess up anything else in the code?
    //this needs to be adressed at some point
    dec_particles[i].setP5(dec_particles[i].M());
    orig_particles.push_back(dec_particles[i]);
  }
  orig_event.add_particles(orig_particles);
  orig_event.setnup(orig_particles.size());

  orig_event.setxwgtup(weight*bratio);
  //cout << decaymode->Parent() <<  " -> " << decaymode->ListDaughters()[0] << ", " 
  //       << decaymode->ListDaughters()[1] << "\n";
  //parent location
  //cout << "location of particle is line number: " << location+1 << endl;       
  //
}
